@@ -5,7 +5,7 @@ use strict;
 use warnings;
 use base qw( Exporter );
 
-our $VERSION = '0.29_01';
+our $VERSION = '0.02';
 
 use AnyEvent::RipeRedis;
 use AnyEvent::RipeRedis::Error;
@@ -15,13 +15,12 @@ use List::MoreUtils qw( bsearch );
 use Scalar::Util qw( looks_like_number weaken );
 use Carp qw( croak );
 
-my %ERROR_CODES;
+our %ERROR_CODES;
 
 BEGIN {
   %ERROR_CODES = %AnyEvent::RipeRedis::Error::ERROR_CODES;
-  my @err_codes  = keys %ERROR_CODES;
-  our @EXPORT_OK = ( @err_codes, qw( crc16 hash_slot ) );
-  our %EXPORT_TAGS = ( err_codes => \@err_codes );
+  our @EXPORT_OK   = keys %ERROR_CODES;
+  our %EXPORT_TAGS = ( err_codes => \@EXPORT_OK, );
 }
 
 use constant {
@@ -72,15 +71,7 @@ my @CRC16_TAB = (
   0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0,
 );
 
-my %PREDEFINED_CMDS = (
-  sort        => { readonly => 0, key_pos => 1 },
-  zunionstore => { readonly => 0, key_pos => 1 },
-  zinterstore => { readonly => 0, key_pos => 1 },
-  eval        => { readonly => 0, movablekeys => 1, key_pos => 0 },
-  evalsha     => { readonly => 0, movablekeys => 1, key_pos => 0 },
-);
-
-my %SUB_CMDS = (
+my %SUB_COMMANDS = (
   subscribe  => 1,
   psubscribe => 1,
 );
@@ -103,8 +94,9 @@ sub new {
   }
 
   $self->{startup_nodes} = $params{startup_nodes};
-  $self->{allow_slaves}  = $params{allow_slaves};
-  $self->{lazy}          = $params{lazy};
+  $self->{allow_slaves}
+      = exists $params{allow_slaves} ? $params{allow_slaves} : 1;
+  $self->{lazy} = $params{lazy};
   $self->refresh_interval( $params{refresh_interval} );
 
   $self->{on_node_connect}    = $params{on_node_connect};
@@ -114,9 +106,8 @@ sub new {
 
   my %node_params;
   foreach my $name ( qw( password utf8 connection_timeout read_timeout
-      reconnect reconnect_interval handle_params ) )
+      reconnect min_reconnect_interval handle_params ) )
   {
-    next unless defined $params{$name};
     $node_params{$name} = $params{$name};
   }
   $self->{_node_params} = \%node_params;
@@ -154,25 +145,6 @@ sub disconnect {
   return;
 }
 
-sub nodes {
-  my $self         = shift;
-  my $key          = shift;
-  my $allow_slaves = shift;
-
-  return unless defined $self->{_slots};
-
-  my $slot;
-  if ( defined $key ) {
-    $slot = hash_slot($key);
-  }
-
-  my $nodes = $self->_nodes( $slot, $allow_slaves );
-
-  return wantarray
-      ? @{ $self->{_nodes_pool} }{ @{$nodes} }
-      : $self->{_nodes_pool}{ $nodes->[0] };
-}
-
 sub refresh_interval {
   my $self = shift;
 
@@ -229,34 +201,20 @@ sub crc16 {
   return $crc;
 }
 
-sub hash_slot {
-  my $hashtag = shift;
-
-  if ( $hashtag =~ m/\{([^}]*?)\}/ ) {
-    if ( length $1 > 0 ) {
-      $hashtag = $1;
-    }
-  }
-
-  return crc16($hashtag) % MAX_SLOTS;
-}
-
 sub _init {
-  my $self = shift;
+  my $self  = shift;
 
   $self->{_init_state} = S_IN_PROGRESS;
   undef $self->{_refresh_timer};
 
   weaken($self);
 
-  $self->_discover_cluster(
+  $self->_discover_nodes(
     sub {
       my $err = $_[1];
 
       if ( defined $err ) {
         $self->{_init_state} = S_NEED_DO;
-
-        $self->{_ready} = 0;
         $self->_abort($err);
 
         return;
@@ -282,123 +240,99 @@ sub _init {
   return;
 }
 
-sub _discover_cluster {
+sub _discover_nodes {
   my $self = shift;
   my $cb   = shift;
 
-  my $nodes;
+  my $nodes_pool = $self->{_nodes_pool};
 
-  if ( defined $self->{_slots} ) {
-    $nodes = $self->_nodes( undef, $self->{allow_slaves} );
+  my @nodes;
+  if ( %{$nodes_pool} ) {
+    @nodes = keys %{$nodes_pool};
   }
   else {
-    my %nodes_pool;
-
     foreach my $node_params ( @{ $self->{startup_nodes} } ) {
       my $hostport = "$node_params->{host}:$node_params->{port}";
 
-      unless ( defined $nodes_pool{$hostport} ) {
-        $nodes_pool{$hostport} = $self->_new_node(
-            $node_params->{host}, $node_params->{port} );
+      unless ( defined $nodes_pool->{$hostport} ) {
+        $nodes_pool->{$hostport} = $self->_new_node(
+            $node_params->{host}, $node_params->{port}, 1 );
       }
-    }
 
-    $self->{_nodes_pool} = \%nodes_pool;
-    $nodes = [ keys %nodes_pool ];
+      push( @nodes, $hostport );
+    }
   }
 
   weaken($self);
 
   $self->_execute(
-    { name => 'cluster_state',
+    { name => 'cluster_slots',
       args => [],
 
       on_reply => sub {
-        my $err = $_[1];
+        my $slots = shift;
+        my $err   = shift;
 
         if ( defined $err ) {
           $cb->( undef, $err );
           return;
         }
 
-        $self->_execute(
-          { name => 'cluster_slots',
-            args => [],
-
-            on_reply => sub {
-              my $slots = shift;
-              my $err   = shift;
-
-              if ( defined $err ) {
-                $cb->( undef, $err );
-                return;
-              }
-
-              unless ( @{$slots} ) {
-                $err = _new_error( 'ERR Returned empty list of slots',
-                    E_UNEXPECTED_DATA );
-                $cb->( undef, $err );
-
-                return;
-              }
-
-              $self->_prepare_nodes( $slots,
-                sub {
-                  unless ( defined $self->{_commands} ) {
-                    $self->_load_commands($cb);
-                    return;
-                  }
-
-                  $cb->();
-                }
-              );
+        $self->_prepare_nodes_pool( $slots,
+          sub {
+            unless ( defined $self->{_commands} ) {
+              $self->_discover_commands($cb);
+              return;
             }
-          },
-          $nodes
+
+            $cb->();
+          }
         );
       }
     },
-    $nodes
+    @nodes,
   );
 
   return;
 }
 
-sub _prepare_nodes {
+sub _prepare_nodes_pool {
   my $self      = shift;
   my $slots_raw = shift;
   my $cb        = shift;
 
   my %nodes_pool;
   my @slots;
-  my @masters_nodes;
-  my @slave_nodes;
+  my @masters;
+  my @slaves;
 
-  my $nodes_pool_old = $self->{_nodes_pool};
+  my $old_nodes_pool = $self->{_nodes_pool};
 
   foreach my $range ( @{$slots_raw} ) {
     my $range_start = shift @{$range};
     my $range_end   = shift @{$range};
 
     my @nodes;
-    my $node_cnt = 0;
+    my $is_master = 1;
 
     foreach my $node_info ( @{$range} ) {
       my $hostport = "$node_info->[0]:$node_info->[1]";
 
       unless ( defined $nodes_pool{$hostport} ) {
-        if ( defined $nodes_pool_old->{$hostport} ) {
-          $nodes_pool{$hostport} = delete $nodes_pool_old->{$hostport};
+        if ( defined $old_nodes_pool->{$hostport} ) {
+          $nodes_pool{$hostport} = $old_nodes_pool->{$hostport};
         }
         else {
           $nodes_pool{$hostport} = $self->_new_node( @{$node_info}[ 0, 1 ] );
+
+          unless ($is_master) {
+            push( @slaves, $hostport );
+          }
         }
 
-        if (++$node_cnt > 1) {
-          push( @slave_nodes, $hostport );
-        }
-        else {
-          push( @masters_nodes, $hostport );
+        if ($is_master) {
+          push( @masters, $hostport );
+          $is_master = 0;
         }
       }
 
@@ -410,17 +344,12 @@ sub _prepare_nodes {
 
   @slots = sort { $a->[0] <=> $b->[0] } @slots;
 
-  foreach my $node ( values %{$nodes_pool_old} ) {
-    $node->disconnect;
-  }
+  $self->{_nodes_pool} = \%nodes_pool;
+  $self->{_masters}    = \@masters;
+  $self->{_slots}      = \@slots;
 
-  $self->{_nodes_pool}   = \%nodes_pool;
-  $self->{_nodes}        = [ keys %nodes_pool ];
-  $self->{_master_nodes} = \@masters_nodes;
-  $self->{_slots}        = \@slots;
-
-  if ( $self->{allow_slaves} && @slave_nodes ) {
-    $self->_prepare_slaves( \@slave_nodes, $cb );
+  if ( $self->{allow_slaves} && @slaves ) {
+    $self->_prepare_slaves( \@slaves, $cb );
     return;
   }
 
@@ -430,11 +359,11 @@ sub _prepare_nodes {
 }
 
 sub _prepare_slaves {
-  my $self        = shift;
-  my $slave_nodes = shift;
-  my $cb          = shift;
+  my $self   = shift;
+  my $slaves = shift;
+  my $cb     = shift;
 
-  my $reply_cnt = scalar @{$slave_nodes};
+  my $reply_cnt = scalar @{$slaves};
 
   my $cmd = {
     name => 'readonly',
@@ -446,20 +375,19 @@ sub _prepare_slaves {
     }
   };
 
-  foreach my $hostport ( @{$slave_nodes} ) {
-    $self->_execute( $cmd, [ $hostport ] );
+  foreach my $hostport ( @{$slaves} ) {
+    $self->_execute( $cmd, $hostport );
   }
 
   return;
 }
 
-sub _load_commands {
+sub _discover_commands {
   my $self = shift;
   my $cb   = shift;
 
-  my $nodes = $self->_nodes( undef, $self->{allow_slaves} );
-
   weaken($self);
+  my @nodes = keys %{ $self->{_nodes_pool} };
 
   $self->_execute(
     { name => 'command',
@@ -474,25 +402,33 @@ sub _load_commands {
           return;
         }
 
-        my %commands = %PREDEFINED_CMDS;
+        my %commands;
 
         foreach my $cmd_raw ( @{$commands_raw} ) {
-          my $kwd = lc( $cmd_raw->[0] );
+          my $kwd     = lc( $cmd_raw->[0] );
+          my $flags   = $cmd_raw->[2];
+          my $key_pos = $cmd_raw->[3];
 
-          next if exists $commands{$kwd};
+          my $cmd_info = {
+            readonly    => 0,
+            movablekeys => 0,
+            key         => $key_pos,
+          };
 
-          my $readonly = 0;
-          foreach my $flag ( @{ $cmd_raw->[2] } ) {
+          foreach my $flag ( @{$flags} ) {
             if ( $flag eq 'readonly' ) {
-              $readonly = 1;
-              last;
+              $cmd_info->{readonly} = 1;
+            }
+            if ( $flag eq 'movablekeys' ) {
+              $cmd_info->{movablekeys} = 1;
             }
           }
 
-          $commands{$kwd} = {
-            readonly => $readonly,
-            key_pos  => $cmd_raw->[3],
-          };
+          $commands{$kwd} = $cmd_info;
+        }
+        $commands{watch}{forcing_slot} = 1;
+        foreach my $kwd ( qw( exec discard unwatch ) ) {
+          $commands{$kwd}{unforcing_slot} = 1;
         }
 
         $self->{_commands} = \%commands;
@@ -500,7 +436,7 @@ sub _load_commands {
         $cb->();
       }
     },
-    $nodes
+    @nodes,
   );
 
   return;
@@ -510,12 +446,13 @@ sub _new_node {
   my $self = shift;
   my $host = shift;
   my $port = shift;
+  my $lazy = shift;
 
   return AnyEvent::RipeRedis->new(
     %{ $self->{_node_params} },
     host          => $host,
     port          => $port,
-    lazy          => 1,
+    lazy          => $lazy,
     on_connect    => $self->_create_on_node_connect( $host, $port ),
     on_disconnect => $self->_create_on_node_disconnect( $host, $port ),
     on_error      => $self->_create_on_node_error( $host, $port ),
@@ -580,7 +517,7 @@ sub _prepare {
   else {
     $cbs = {};
     if ( ref( $args->[-1] ) eq 'CODE' ) {
-      if ( exists $SUB_CMDS{$cmd_name} ) {
+      if ( exists $SUB_COMMANDS{$cmd_name} ) {
         $cbs->{on_message} = pop @{$args};
       }
       else {
@@ -603,13 +540,14 @@ sub _prepare {
 
   unless ( defined $cmd->{on_reply} ) {
     $cmd->{on_reply} = sub {
-      my $err = $_[1];
+      my $reply = shift;
+      my $err   = shift;
 
       if ( defined $err ) {
-        $self->{on_error}->($err);
+        $self->{on_error}->( $err, $reply );
         return;
       }
-    };
+    }
   }
 
   return $cmd;
@@ -623,123 +561,130 @@ sub _route {
     if ( $self->{_init_state} == S_NEED_DO ) {
       $self->_init;
     }
+
     push( @{ $self->{_input_queue} }, $cmd );
 
     return;
   }
 
-  my $key;
-  my $kwds = $cmd->{kwds};
-  my $args = $cmd->{args};
-  my $cmd_info = $self->{_commands}{ $kwds->[0] };
-
-  if ( defined $cmd_info ) {
-    if ( $cmd_info->{key_pos} > 0 ) {
-      $key = $args->[ $cmd_info->{key_pos} - scalar @{$kwds} ];
-    }
-    # Exception for EVAL and EVALSHA commands
-    elsif ( $cmd_info->{movablekeys}
-      && $args->[1] > 0 )
-    {
-      $key = $args->[2];
-    }
+  if ( $cmd->{name} eq 'multi'
+    && !defined $self->{_forced_slot} )
+  {
+    $self->{_deferred_multi} = $cmd;
+    return;
   }
 
-  my $slot;
-  my $allow_slaves = $self->{allow_slaves};
+  weaken($self);
 
-  if ( defined $key ) {
-    $slot = hash_slot($key);
-    $allow_slaves &&= $cmd_info->{readonly};
-  }
+  $self->_get_route( $cmd,
+    sub {
+      my $slot         = shift;
+      my $allow_slaves = shift;
 
-  my $nodes = $self->_nodes( $slot, $allow_slaves );
-  $self->_execute( $cmd, $nodes );
+      my @nodes;
+
+      if ( defined $slot ) {
+        if ( defined $self->{_deferred_multi} ) {
+          my $multi = $self->{_deferred_multi};
+          undef $self->{_deferred_multi};
+          $self->_route($multi);
+
+          $self->_route($cmd);
+
+          return;
+        }
+
+        my ($range) = bsearch {
+          $slot > $_->[1] ? -1 : $slot < $_->[0] ? 1 : 0;
+        }
+        @{ $self->{_slots} };
+
+        @nodes
+            = $allow_slaves
+            ? @{ $range->[2] }
+            : ( $range->[2][0] );
+      }
+      else {
+        @nodes
+            = $allow_slaves
+            ? keys %{ $self->{_nodes_pool} }
+            : @{ $self->{_masters} };
+      }
+
+      $self->_execute( $cmd, @nodes );
+    }
+  );
 
   return;
 }
 
 sub _execute {
-  my $self       = shift;
-  my $cmd        = shift;
-  my $nodes      = shift;
-  my $node_index = shift;
-  my $fails_cnt  = shift || 0;
+  my $self  = shift;
+  my $cmd   = shift;
+  my @nodes = @_;
 
-  unless ( defined $node_index ) {
-    $node_index = int( rand( scalar @{$nodes} ) );
+  my $hostport;
+  if ( scalar @nodes > 1 ) {
+    my $rand_index = int( rand( scalar @nodes ) );
+    $hostport = splice( @nodes, $rand_index, 1 );
   }
-  elsif ( $node_index == scalar @{$nodes} ) {
-    $node_index = 0;
+  else {
+    $hostport = shift @nodes;
   }
-  my $hostport = $nodes->[$node_index];
-  my $node     = $self->{_nodes_pool}{$hostport};
-
-  my $cmd_name = $cmd->{name} eq 'cluster_state'
-      ? 'cluster_info'
-      : $cmd->{name};
+  my $node = $self->{_nodes_pool}{$hostport};
 
   weaken($self);
 
-  $node->execute( $cmd_name, @{ $cmd->{args} },
-    { on_reply => sub {
-        my $reply = shift;
-        my $err   = shift;
+  my $on_reply = sub {
+    my $reply = shift;
+    my $err   = shift;
 
-        if ( $cmd->{name} eq 'cluster_state' ) {
-          unless ( defined $err ) {
-            if ( $reply->{cluster_state} eq 'ok' ) {
-              $reply = 1;
-            }
-            else {
-              $err = _new_error( 'CLUSTERDOWN The cluster is down',
-                  E_CLUSTER_DOWN );
-            }
-          }
+    if ( defined $err ) {
+      my $err_code   = $err->code;
+      my $nodes_pool = $self->{_nodes_pool};
+
+      if ( $err_code == E_MOVED || $err_code == E_ASK ) {
+        if ( $err_code == E_MOVED ) {
+          $self->{_init_state} = S_NEED_DO;
+          $self->{_ready}      = 0;
         }
 
-        if ( defined $err ) {
-          my $err_code   = $err->code;
-          my $nodes_pool = $self->{_nodes_pool};
+        my ($fwd_hostport) = ( split( m/\s+/, $err->message ) )[2];
 
-          if ( $err_code == E_MOVED || $err_code == E_ASK ) {
-            if ( $err_code == E_MOVED ) {
-              $self->{_init_state} = S_NEED_DO;
-              $self->{_ready}      = 0;
-            }
-
-            my ($fwd_hostport) = ( split( m/\s+/, $err->message ) )[2];
-
-            unless ( defined $nodes_pool->{$fwd_hostport} ) {
-              my ( $host, $port ) = parse_hostport($fwd_hostport);
-              $nodes_pool->{$fwd_hostport} = $self->_new_node( $host, $port );
-            }
-
-            $self->_execute( $cmd, [ $fwd_hostport ] );
-
-            return;
-          }
-
-          my $on_node_error = $cmd->{on_node_error} || $self->{on_node_error};
-          if ( defined $on_node_error ) {
-            my $node = $nodes_pool->{$hostport};
-            $on_node_error->( $err, $node->host, $node->port );
-          }
-
-          if ( $err_code != E_CONN_CLOSED_BY_CLIENT
-            && ++$fails_cnt < scalar @{$nodes} )
-          {
-            $self->_execute( $cmd, $nodes, ++$node_index, $fails_cnt );
-            return;
-          }
-
-          $cmd->{on_reply}->( $reply, $err );
-
-          return;
+        unless ( defined $nodes_pool->{$fwd_hostport} ) {
+          my ( $host, $port ) = parse_hostport($fwd_hostport);
+          $nodes_pool->{$fwd_hostport} = $self->_new_node( $host, $port );
         }
 
-        $cmd->{on_reply}->($reply);
-      },
+        $self->_execute( $cmd, $fwd_hostport );
+
+        return;
+      }
+
+      my $on_node_error = $cmd->{on_node_error} || $self->{on_node_error};
+
+      if ( defined $on_node_error ) {
+        my $node = $nodes_pool->{$hostport};
+        $on_node_error->( $err, $node->host, $node->port );
+      }
+
+      if ( $err_code != E_CONN_CLOSED_BY_CLIENT && @nodes ) {
+        $self->_execute( $cmd, @nodes );
+        return;
+      }
+
+      $cmd->{on_reply}->( $reply, $err );
+
+      return;
+    }
+
+    $cmd->{on_reply}->($reply);
+
+    return;
+  };
+
+  $node->execute( $cmd->{name}, @{ $cmd->{args} },
+    { on_reply => $on_reply,
 
       defined $cmd->{on_message}
       ? ( on_message => $cmd->{on_message} )
@@ -750,25 +695,105 @@ sub _execute {
   return;
 }
 
-sub _nodes {
+sub _get_route {
   my $self = shift;
-  my $slot = shift;
-  my $allow_slaves = shift;
+  my $cmd  = shift;
+  my $cb   = shift;
 
-  if ( defined $slot ) {
-    my ($range) = bsearch {
-      $slot > $_->[1] ? -1 : $slot < $_->[0] ? 1 : 0;
+  my $cmd_info = $self->{_commands}{ $cmd->{kwds}[0] };
+
+  if ( defined $self->{_forced_slot} ) {
+    my $slot = $self->{_forced_slot};
+
+    if ( defined $cmd_info && $cmd_info->{unforcing_slot} ) {
+      undef $self->{_forced_slot};
     }
-    @{ $self->{_slots} };
 
-    return $allow_slaves
-        ? $range->[2]
-        : [ $range->[2][0] ];
+    $cb->($slot);
+
+    return;
   }
 
-  return $allow_slaves
-      ? $self->{_nodes}
-      : $self->{_master_nodes};
+  $self->_get_key( $cmd,
+    sub {
+      my $key = shift;
+
+      unless ( defined $key ) {
+        $cb->( undef, $self->{allow_slaves} );
+        return;
+      }
+
+      my $slot = _get_slot($key);
+
+      if ( ( defined $cmd_info && $cmd_info->{forcing_slot} )
+        || defined $self->{_deferred_multi} )
+      {
+        $self->{_forced_slot} = $slot;
+      }
+
+      $cb->( $slot, $self->{allow_slaves} && $cmd_info->{readonly} );
+    }
+  );
+
+  return;
+}
+
+sub _get_key {
+  my $self = shift;
+  my $cmd  = shift;
+  my $cb   = shift;
+
+  my $cmd_info = $self->{_commands}{ $cmd->{kwds}[0] };
+
+  unless ( defined $cmd_info ) {
+    $cb->();
+    return;
+  }
+
+  my @args = ( @{ $cmd->{kwds} }, @{ $cmd->{args} } );
+
+  if ( $cmd_info->{movablekeys} ) {
+    my @nodes
+        = $self->{allow_slaves}
+        ? keys %{ $self->{_nodes_pool} }
+        : @{ $self->{_masters} };
+
+    $self->_execute(
+      { name          => 'command_getkeys',
+        args          => [@args],
+        on_node_error => $cmd->{on_node_error},
+
+        on_reply => sub {
+          my $keys = shift;
+          my $err  = shift;
+
+          if ( defined $err ) {
+            $cmd->{on_reply}->( undef, $err );
+            return;
+          }
+
+          if ( @{$keys} ) {
+            $cb->( $keys->[0] );
+          }
+          else {
+            $cb->();
+          }
+        }
+      },
+      @nodes,
+    );
+
+    return;
+  }
+
+  if ( $cmd_info->{key} > 0 ) {
+    $cb->( $args[ $cmd_info->{key} ] );
+  }
+  else {
+    $cb->();
+  }
+
+  return;
 }
 
 sub _process_input_queue {
@@ -787,14 +812,15 @@ sub _process_input_queue {
 sub _reset_internals {
   my $self = shift;
 
-  $self->{_nodes_pool}    = undef;
-  $self->{_nodes}         = undef;
-  $self->{_master_nodes}  = undef;
-  $self->{_slots}         = undef;
-  $self->{_commands}      = undef;
-  $self->{_init_state}    = S_NEED_DO;
-  $self->{_refresh_timer} = undef;
-  $self->{_ready}         = 0;
+  $self->{_nodes_pool}     = {};
+  $self->{_masters}        = undef;
+  $self->{_slots}          = undef;
+  $self->{_commands}       = undef;
+  $self->{_init_state}     = S_NEED_DO;
+  $self->{_refresh_timer}  = undef;
+  $self->{_ready}          = 0;
+  $self->{_deferred_multi} = undef;
+  $self->{_forced_slot}    = undef;
 
   return;
 }
@@ -839,6 +865,20 @@ sub _queued_commands {
   );
 }
 
+sub _get_slot {
+  my $key = shift;
+
+  my $tag = $key;
+
+  if ( $key =~ m/\{([^}]*?)\}/ ) {
+    if ( length $1 > 0 ) {
+      $tag = $1;
+    }
+  }
+
+  return crc16($tag) % MAX_SLOTS;
+}
+
 sub _new_error {
   return AnyEvent::RipeRedis::Error->new(@_);
 }
@@ -896,7 +936,6 @@ AnyEvent::RipeRedis::Cluster - Non-blocking Redis Cluster client
     startup_nodes => [
       { host => 'localhost', port => 7000 },
       { host => 'localhost', port => 7001 },
-      { host => 'localhost', port => 7002 },
     ],
   );
 
@@ -963,40 +1002,26 @@ L<http://redis.io/topics/cluster-spec>
     startup_nodes => [
       { host => 'localhost', port => 7000 },
       { host => 'localhost', port => 7001 },
-      { host => 'localhost', port => 7002 },
     ],
-    password           => 'yourpass',
-    connection_timeout => 5,
-    read_timeout       => 5,
-    refresh_interval   => 5,
-    lazy               => 1,
-    reconnect_interval => 5,
+    connection_timeout     => 5,
+    read_timeout           => 5,
+    refresh_interval       => 5,
+    lazy                   => 1,
+    min_reconnect_interval => 5,
 
     on_node_connect => sub {
-      my $host = shift;
-      my $port = shift;
-
       # handling...
     },
 
     on_node_disconnect => sub {
-      my $host = shift;
-      my $port = shift;
-
       # handling...
     },
 
     on_node_error => sub {
-      my $err = shift;
-      my $host = shift;
-      my $port = shift;
-
       # error handling...
     },
 
     on_error => sub {
-      my $err = shift;
-
       # error handling...
     },
   );
@@ -1005,11 +1030,11 @@ L<http://redis.io/topics/cluster-spec>
 
 =item startup_nodes => \@nodes
 
-Specifies the list of startup nodes. Parameter should contain the array of
-hashes that contains addresses of some nodes in the cluster. Each hash should
-contain C<host> and C<port> elements. The client will try to connect to random
-node from the list to retrieve information about all cluster nodes and slots
-mapping. If the client could not connect to first selected node, it will try
+Specifies the list of startup nodes. Parameter should contain array of hashes
+that contains addresses of some nodes in the cluster. Each hash should contain
+C<host> and C<port> elements. The client will try to connect to random node
+from the list to retrieve information about all cluster nodes and slots
+mappings. If the client could not connect to first selected node, it will try
 to connect to another random node from the list.
 
 =item password => $password
@@ -1021,15 +1046,11 @@ of the cluster after connection.
 
 If enabled, the client will try to send read-only commands to slave nodes.
 
-Disabled by default.
-
 =item utf8 => $boolean
 
 If enabled, all strings will be converted to UTF-8 before sending to nodes,
 and all results will be decoded from UTF-8.
 
-Enabled by default.
-
 =item connection_timeout => $fractional_seconds
 
 Specifies connection timeout. If the client could not connect to the node
@@ -1044,9 +1065,9 @@ By default the client use kernel's connection timeout.
 =item read_timeout => $fractional_seconds
 
 Specifies read timeout. If the client could not receive a reply from the node
-after specified timeout, the client close connection and call the
-C<on_node_error> callback with the C<E_READ_TIMEDOUT> error. The timeout is
-specifies in seconds and can contain a fractional part.
+after specified timeout, the client close connection and call the C<on_error>
+callback with the C<E_READ_TIMEDOUT> error. The timeout is specifies in seconds
+and can contain a fractional part.
 
   read_timeout => 3.5,
 
@@ -1054,9 +1075,9 @@ Not set by default.
 
 =item lazy => $boolean
 
-If enabled, the initial connection to the startup node establishes at time when
-you will send the first command to the cluster. By default the initial
-connection establishes after calling of the C<new> method.
+If enabled, the connection to nodes establishes at time when you will send the
+first command to the cluster. By default the connection establishes after
+calling of the C<new> method.
 
 Disabled by default.
 
@@ -1071,12 +1092,12 @@ you need. Such behavior allows to control reconnection procedure.
 
 Enabled by default.
 
-=item reconnect_interval => $fractional_seconds
+=item min_reconnect_interval => $fractional_seconds
 
-If the parameter is specified, the client will try to reconnect only after
-this interval. Commands executed between reconnections will be queued.
+If the parameter is specified, the client will try to reconnect not often, than
+after this interval. Commands executed between reconnections will be queued.
 
-  reconnect_interval => 5,
+  min_reconnect_interval => 5,
 
 Not set by default.
 
@@ -1096,20 +1117,20 @@ Specifies L<AnyEvent::Handle> parameters.
     linger   => 60,
   }
 
-Enabling of the C<autocork> parameter can improve performance. See
+Enabling of the C<autocork> parameter can improve perfomance. See
 documentation on L<AnyEvent::Handle> for more information.
 
 =item on_node_connect => $cb->( $host, $port )
 
-The C<on_node_connect> callback is called when the connection to particular
-node is successfully established. To callback are passed two arguments: host
-and port of the node to which the client was connected.
+The C<on_node_connect> callback is called when the connection to specific node
+is successfully established. To callback are passed two arguments: host and
+port of the node to which the client was connected.
 
 Not set by default.
 
 =item on_node_disconnect => $cb->( $host, $port )
 
-The C<on_node_disconnect> callback is called when the connection to particular
+The C<on_node_disconnect> callback is called when the connection to specific
 node is closed by any reason. To callback are passed two arguments: host and
 port of the node from which the client was disconnected.
 
@@ -1139,18 +1160,17 @@ to C<STDERR>.
 
 =head2 <command>( [ @args ] [, ( $cb->( $reply, $err ) | \%cbs ) ] )
 
-To execute the command you must call particular method with corresponding name.
+To execute the command you must call specific method with corresponding name.
 The reply to the command is passed to the callback in first argument. If any
 error occurred during the command execution, the error object is passed to the
-callback in second argument. The error object is the instance of the class
+callback in second argument. Error object is an instance of the class
 L<AnyEvent::RipeRedis::Error>.
 
 Before the command execution, the client determines the pool of nodes, on which
 the command can be executed. The pool can contain the one or more nodes
-depending on the cluster and the client configurations, and the command type.
-The client will try to execute the command on random node from the pool and, if
-the command failed on selected node, the client will try to execute it on
-another random node.
+depending on the cluster configuration and command type. The client will try to
+execute the command on random node from the pool and, if the command failed on
+selected node, the client will try to execute it on another random node.
 
 The command callback is optional. If it is not specified and any error
 occurred, the C<on_error> callback of the client is called.
@@ -1197,7 +1217,7 @@ The full list of the Redis commands can be found here: L<http://redis.io/command
 
   $cluster->incr( 'counter' );
 
-If you want to track errors on particular nodes, you must specify C<on_node_error>
+If you want to track errors on specific nodes, you must specify C<on_node_error>
 callback in command method.
 
   $cluster->get( 'foo',
@@ -1227,7 +1247,7 @@ callback in command method.
     }
   );
 
-=head2 execute( $command [, @args ] [, ( $cb->( $reply, $err ) | \%cbs ) ] )
+=head2 execute( $command, [ @args ] [, ( $cb->( $reply, $err ) | \%cbs ) ] )
 
 An alternative method to execute commands. In some cases it can be more
 convenient.
@@ -1250,36 +1270,6 @@ convenient.
     }
   );
 
-=head1 TRANSACTIONS
-
-To perform the transaction you must get the master node by the key using
-C<nodes> method and then execute all commands on this node. Nodes must be
-discovered first.
-
-  $node = $cluster->nodes('foo');
-
-  $node->multi;
-  $node->set( '{foo}bar', "some\r\nstring" );
-  $node->set( '{foo}car', 42 );
-  $node->exec(
-    sub {
-      my $reply = shift;
-      my $err   = shift;
-
-      if ( defined $err ) {
-        # error handling...
-
-        return;
-      }
-
-      # reply handling...
-    }
-  );
-
-The detailed information about the Redis transactions can be found in
-documentation on L<AnyEvent::RipeRedis> and here:
-L<http://redis.io/topics/transactions>.
-
 =head1 ERROR CODES
 
 Every error object, passed to callback, contain error code, which can be used
@@ -1305,50 +1295,21 @@ aborted.
 
 =head1 OTHER METHODS
 
-=head2 nodes( [ $key ] [, $allow_slaves ] )
-
-Gets particular nodes of the cluster. Nodes must be discovered first. In scalar
-context method returns the first node from the list.
-
-Getting all master nodes of the cluster:
-
-  my @master_nodes = $cluster->nodes;
-
-Getting all nodes of the cluster, including slave nodes:
-
-  my @nodes = $cluster->nodes( undef, 1 );
-
-Getting master node by the key:
-
-  my $master_node = $cluster->nodes('foo');
-
-Getting nodes by the key, including slave nodes:
-
-  my @nodes = $cluster->nodes( 'foo', 1 );
-
 =head2 refresh_interval( [ $fractional_seconds ] )
 
-Gets or sets the C<refresh_interval> of the client. The C<undef> value resets
+Get or set the C<refresh_interval> of the client. The C<undef> value resets
 the C<refresh_interval> to default value.
 
 =head2 on_error( [ $callback ] )
 
-Gets or sets the C<on_error> callback.
+Get or set the C<on_error> callback.
 
 =head1 SERVICE FUNCTIONS
 
-Service functions provided by AnyEvent::RipeRedis::Cluster can be imported.
-
-  use AnyEvent::RipeRedis::Cluster qw( crc16 hash_slot );
-
 =head2 crc16( $data )
 
 Compute CRC16 for the specified data as defined in Redis Cluster specification.
 
-=head2 hash_slot( $key );
-
-Returns slot number by the key.
-
 =head1 SEE ALSO
 
 L<AnyEvent::RipeRedis>
@@ -1361,7 +1322,8 @@ Sponsored by SMS Online, E<lt>dev.opensource@sms-online.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2016-2017, Eugene Ponizovsky, SMS Online. All rights reserved.
+Copyright (c) 2016, Eugene Ponizovsky, E<lt>ponizovsky@gmail.comE<gt>,
+SMS Online, E<lt>dev.opensource@sms-online.comE<gt>. All rights reserved.
 
 This module is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
